/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.examples.groups;

import com.alee.examples.WebLookAndFeelDemo;
import com.alee.laf.label.WebLabel;
import com.alee.laf.panel.WebPanel;
import com.alee.managers.tooltip.TooltipManager;
import com.alee.managers.tooltip.TooltipWay;
import com.alee.managers.tooltip.WebCustomTooltip;
import com.alee.utils.WebUtils;

import javax.swing.*;
import java.awt.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.geom.Ellipse2D;

/**
 * User: mgarin Date: 26.01.12 Time: 14:02
 */

public class ImagePanel extends WebPanel
{
    private static ImageIcon logoIcon =
            new ImageIcon ( ImagePanel.class.getResource ( "icons/logo.png" ) );
    private static ImageIcon linkIcon =
            new ImageIcon ( ImagePanel.class.getResource ( "icons/link.png" ) );

    public ImagePanel ()
    {
        super ();

        setOpaque ( false );

        MouseAdapter mouseAdapter = new MouseAdapter ()
        {
            boolean inside = false;

            public void mouseEntered ( MouseEvent e )
            {
                if ( getShape ().contains ( e.getPoint () ) )
                {
                    inside = true;
                    updateCursor ();
                }
            }

            public void mouseExited ( MouseEvent e )
            {
                if ( inside )
                {
                    inside = false;
                    updateCursor ();
                }
            }

            public void mouseMoved ( MouseEvent e )
            {
                boolean contains = getShape ().contains ( e.getPoint () );
                if ( !inside && contains )
                {
                    inside = true;
                    updateCursor ();
                }
                else if ( inside && !contains )
                {
                    inside = false;
                    updateCursor ();
                }
            }

            public void mousePressed ( MouseEvent e )
            {
                if ( inside )
                {
                    WebUtils.browseSiteSafely ( WebLookAndFeelDemo.WEBLAF_SITE );
                }
            }

            private Shape getShape ()
            {
                return new Ellipse2D.Double ( 10 + 5,
                        getHeight () - logoIcon.getIconHeight () - 10 + 5,
                        logoIcon.getIconWidth () - 10, logoIcon.getIconHeight () - 10 );
            }

            private WebCustomTooltip tip = null;
            private WebLabel tipLabel = null;

            private void updateCursor ()
            {
                if ( inside )
                {
                    if ( tipLabel == null )
                    {
                        tipLabel = new WebLabel ( WebLookAndFeelDemo.WEBLAF_SITE, linkIcon );
                    }

                    setCursor ( Cursor.getPredefinedCursor ( Cursor.HAND_CURSOR ) );
                    Point p = new Point ( 10 + logoIcon.getIconWidth (),
                            ImagePanel.this.getHeight () - 10 -
                                    logoIcon.getIconHeight () / 2 );
                    this.tip = TooltipManager
                            .showOneTimeTooltip ( ImagePanel.this, p, tipLabel, TooltipWay.right );
                }
                else
                {
                    setCursor ( Cursor.getDefaultCursor () );
                    if ( tip != null )
                    {
                        if ( tip.isShowing () )
                        {
                            tip.closeTooltip ();
                        }
                        tip.destroyTooltip ();
                        tip = null;
                    }
                }
            }
        };
        addMouseListener ( mouseAdapter );
        addMouseMotionListener ( mouseAdapter );
    }

    protected void paintComponent ( Graphics g )
    {
        super.paintComponent ( g );
        drawLogo ( g, ImagePanel.this );
    }

    public static void drawLogo ( Graphics g, JComponent component )
    {
        Graphics2D g2d = ( Graphics2D ) g;
        g2d.setRenderingHint ( RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON );

        g2d.drawImage ( logoIcon.getImage (), 10,
                component.getHeight () - logoIcon.getIconHeight () - 10, null );
    }
}