/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.examples.groups;

import com.alee.laf.StyleConstants;
import com.alee.laf.button.WebButton;
import com.alee.laf.label.WebLabel;
import com.alee.laf.panel.WebPanel;
import com.alee.laf.tabbedpane.WebTabbedPane;
import com.alee.laf.toolbar.ToolbarStyle;
import com.alee.laf.toolbar.WebToolBar;
import com.alee.utils.TextUtils;
import com.alee.utils.reflection.JarEntry;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.List;

/**
 * User: mgarin Date: 06.03.12 Time: 13:32
 */

public class ViewTabbedPane extends WebTabbedPane
{
    public static final ImageIcon REMOVE_ICON =
            new ImageIcon ( ViewTabbedPane.class.getResource ( "icons/remove.png" ) );

    public static final String ID_PREFIX = "VTP";

    private List<ViewListener> viewListeners = new ArrayList<ViewListener> ();

    private List<String> ids = new ArrayList<String> ();
    private List<JarEntry> data = new ArrayList<JarEntry> ();

    public ViewTabbedPane ()
    {
        super ( WebTabbedPane.TOP );
        setTabInsets ( new Insets ( 2, 3, 1, 1 ) );

        addMouseListener ( new MouseAdapter ()
        {
            public void mousePressed ( MouseEvent e )
            {
                if ( SwingUtilities.isMiddleMouseButton ( e ) )
                {
                    int index =
                            getUI ().tabForCoordinate ( ViewTabbedPane.this, e.getX (), e.getY () );
                    if ( index != -1 )
                    {
                        removeTabAt ( index );
                    }
                }
            }
        } );
    }

    public void viewEntry ( JarEntry entry, Component viewer )
    {
        if ( data.contains ( entry ) )
        {
            setSelectedIndex ( data.indexOf ( entry ) );
        }
        else
        {
            WebPanel content = new WebPanel ();
            WebToolBar toolBar = new WebToolBar ();
            toolBar.setToolbarStyle ( ToolbarStyle.attached );
            toolBar.setShadeWidth ( 0 );
            content.add ( toolBar, BorderLayout.NORTH );
            content.add ( viewer, BorderLayout.CENTER );

            //            MatteBorder b1 = BorderFactory.createMatteBorder ( 1, 0, 0, 0, Color.WHITE );
            //            MatteBorder b2 =
            //                    BorderFactory.createMatteBorder ( 1, 0, 0, 0, StyleConstants.darkBorderColor );
            //            BorderPanel bp =
            //                    new BorderPanel ( viewer, BorderFactory.createCompoundBorder ( b1, b2 ) );

            int index = getSelectedIndex () + 1;
            insertTab ( entry.getName (), entry.getIcon (), content, null, index );

            setSelectedIndex ( index );
            data.add ( entry );

            fireViewOpened ( data.get ( index ) );
        }
    }

    public void closeEntryView ( JarEntry entry )
    {
        if ( data.contains ( entry ) )
        {
            removeTabAt ( data.indexOf ( entry ) );
        }
    }

    public JarEntry getSelectedEntry ()
    {
        int index = getSelectedIndex ();
        return index != -1 ? data.get ( index ) : null;
    }

    public void insertTab ( String title, Icon icon, Component component, String tip, int index )
    {
        super.insertTab ( title, icon, component, tip, index );

        String id = TextUtils.generateId ( ID_PREFIX );
        setTabComponentAt ( index, createRemovableTitle ( title, icon, id ) );
        ids.add ( index, id );
    }

    private Component createRemovableTitle ( String title, Icon icon, final String id )
    {
        WebPanel removableTitle = new WebPanel ();
        removableTitle.setOpaque ( false );

        WebLabel titleLabel = new WebLabel ( title, icon );
        titleLabel.setMargin ( 0, 2, 0, 4 );
        removableTitle.add ( titleLabel, BorderLayout.CENTER );

        WebButton remove = WebButton
                .createIconWebButton ( REMOVE_ICON, StyleConstants.smallRound, 2, 2, true );
        remove.setFocusable ( false );
        remove.addActionListener ( new ActionListener ()
        {
            public void actionPerformed ( ActionEvent e )
            {
                removeTabAt ( ids.indexOf ( id ) );
            }
        } );
        removableTitle.add ( remove, BorderLayout.EAST );

        return removableTitle;
    }

    public void removeTabAt ( int index )
    {
        super.removeTabAt ( index );

        JarEntry removed = data.get ( index );
        ids.remove ( index );
        data.remove ( index );
        fireViewClosed ( removed );
    }

    public List<ViewListener> getViewListeners ()
    {
        return viewListeners;
    }

    public void setViewListeners ( List<ViewListener> viewListeners )
    {
        this.viewListeners = viewListeners;
    }

    public void addViewListener ( ViewListener listener )
    {
        this.viewListeners.add ( listener );
    }

    public void removeViewListener ( ViewListener listener )
    {
        this.viewListeners.remove ( listener );
    }

    private void fireViewOpened ( JarEntry entry )
    {
        List<ViewListener> viewListenersC = new ArrayList<ViewListener> ();
        viewListenersC.addAll ( viewListeners );
        for ( ViewListener listener : viewListenersC )
        {
            listener.viewOpened ( entry );
        }
    }

    private void fireViewClosed ( JarEntry entry )
    {
        List<ViewListener> viewListenersC = new ArrayList<ViewListener> ();
        viewListenersC.addAll ( viewListeners );
        for ( ViewListener listener : viewListenersC )
        {
            listener.viewClosed ( entry );
        }
    }
}