/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.examples.groups.collapsiblepane;

import com.alee.examples.DefaultExample;
import com.alee.examples.FeatureState;
import com.alee.examples.WebLookAndFeelDemo;
import com.alee.examples.groups.ExamplesManager;
import com.alee.extended.panel.GroupPanel;
import com.alee.extended.panel.WebCollapsiblePane;
import com.alee.laf.scroll.WebScrollPane;
import com.alee.laf.text.WebTextArea;

import javax.swing.*;
import java.awt.*;

/**
 * User: mgarin Date: 20.02.12 Time: 15:26
 */

public class CollapsiblePanesExample extends DefaultExample
{
    public static final ImageIcon textIcon =
            new ImageIcon ( CollapsiblePanesExample.class.getResource ( "icons/text.png" ) );

    public String getTitle ()
    {
        return "Collapsible panes";
    }

    public String getDescription ()
    {
        return "Web-styled collapsible panes";
    }

    public FeatureState getFeatureState ()
    {
        return FeatureState.alpha;
    }

    public Component getPreview ( WebLookAndFeelDemo owner )
    {
        WebCollapsiblePane collapsiblePane1 =
                new WebCollapsiblePane ( "Sample panel 1", textIcon, createCustomContent () );

        WebCollapsiblePane collapsiblePane2 =
                new WebCollapsiblePane ( "Sample panel 2", textIcon, createCustomContent () );
        collapsiblePane2.setExpanded ( false, false );

        WebCollapsiblePane collapsiblePane3 =
                new WebCollapsiblePane ( "Sample panel 3", textIcon, createCustomContent () );
        collapsiblePane3.setExpanded ( false, false );

        return new GroupPanel ( 4, false, collapsiblePane1, collapsiblePane2, collapsiblePane3 );
    }

    private WebScrollPane createCustomContent ()
    {
        WebTextArea textArea = new WebTextArea ( ExamplesManager.createLongString () );
        textArea.setLineWrap ( true );
        textArea.setWrapStyleWord ( true );
        WebScrollPane scrollPane = new WebScrollPane ( textArea, false );
        scrollPane.setPreferredSize ( new Dimension ( 250, 100 ) );
        return scrollPane;
    }
}