/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.examples.groups.desktoppane;

import com.alee.examples.DefaultExample;
import com.alee.examples.FeatureState;
import com.alee.examples.WebLookAndFeelDemo;
import com.alee.examples.groups.desktoppane.tetris.Tetris;
import com.alee.examples.groups.desktoppane.tetris.TetrisListener;
import com.alee.laf.button.WebButton;
import com.alee.laf.desktoppane.WebDesktopPane;
import com.alee.laf.desktoppane.WebInternalFrame;
import com.alee.utils.ThreadUtils;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.beans.PropertyVetoException;

/**
 * User: mgarin Date: 16.02.12 Time: 16:14
 */

public class DesktopPaneExample extends DefaultExample
{
    public static final ImageIcon loader =
            new ImageIcon ( DesktopPaneExample.class.getResource ( "icons/loader.gif" ) );

    public static final ImageIcon frame =
            new ImageIcon ( DesktopPaneExample.class.getResource ( "icons/frame.png" ) );
    public static final ImageIcon game =
            new ImageIcon ( DesktopPaneExample.class.getResource ( "icons/game.png" ) );

    public static final ImageIcon frame32 =
            new ImageIcon ( DesktopPaneExample.class.getResource ( "icons/webframe.png" ) );
    public static final ImageIcon tetris32 =
            new ImageIcon ( DesktopPaneExample.class.getResource ( "icons/tetris.png" ) );

    public static final ImageIcon newGame =
            new ImageIcon ( DesktopPaneExample.class.getResource ( "icons/tetris/new.png" ) );
    public static final ImageIcon pauseGame =
            new ImageIcon ( DesktopPaneExample.class.getResource ( "icons/tetris/pause.png" ) );
    public static final ImageIcon unpauseGame =
            new ImageIcon ( DesktopPaneExample.class.getResource ( "icons/tetris/unpause.png" ) );
    public static final ImageIcon exitGame =
            new ImageIcon ( DesktopPaneExample.class.getResource ( "icons/tetris/exit.png" ) );

    public String getTitle ()
    {
        return "Desktop pane";
    }

    public String getDescription ()
    {
        return "Web-styled desktop pane";
    }

    public FeatureState getFeatureState ()
    {
        return FeatureState.beta;
    }

    public Component getPreview ( WebLookAndFeelDemo owner )
    {
        final WebDesktopPane desktopPane = new WebDesktopPane ();
        desktopPane.setOpaque ( false );

        // Simple frame

        final WebInternalFrame webInternalFrame =
                new WebInternalFrame ( "Web frame", true, true, true, true );
        webInternalFrame.setFrameIcon ( frame );
        webInternalFrame.add ( new JLabel ( "Just an empty frame", JLabel.CENTER )
        {
            {
                setOpaque ( false );
            }
        } );
        desktopPane.add ( webInternalFrame );

        // Simple frame icon

        final WebButton internalFrameIcon = new WebButton ( "Web frame", frame32 );
        internalFrameIcon.setRolloverDecoratedOnly ( true );
        internalFrameIcon.setHorizontalTextPosition ( WebButton.CENTER );
        internalFrameIcon.setVerticalTextPosition ( WebButton.BOTTOM );
        internalFrameIcon.addActionListener ( new ActionListener ()
        {
            public void actionPerformed ( ActionEvent e )
            {
                if ( internalFrameIcon.getClientProperty ( ObjectMoveAdapter.DRAGGED_MARK ) !=
                        null )
                {
                    return;
                }

                if ( webInternalFrame.isClosed () )
                {
                    try
                    {
                        if ( webInternalFrame.getParent () == null )
                        {
                            desktopPane.add ( webInternalFrame );
                        }
                        webInternalFrame.setClosed ( false );
                        webInternalFrame.setVisible ( true );
                        webInternalFrame.setIcon ( false );
                    }
                    catch ( PropertyVetoException e1 )
                    {
                        //
                    }
                }
                else
                {
                    try
                    {
                        webInternalFrame.setIcon ( !webInternalFrame.isIcon () );
                    }
                    catch ( PropertyVetoException e1 )
                    {
                        //
                    }
                }
            }
        } );
        ObjectMoveAdapter ma1 = new ObjectMoveAdapter ();
        internalFrameIcon.addMouseListener ( ma1 );
        internalFrameIcon.addMouseMotionListener ( ma1 );
        internalFrameIcon.setBounds ( 25, 125, 100, 75 );
        desktopPane.add ( internalFrameIcon );

        webInternalFrame.setBounds ( 25 + 100 + 50, 50, 300, 300 );
        webInternalFrame.setVisible ( true );
        try
        {
            webInternalFrame.setClosed ( true );
        }
        catch ( PropertyVetoException e )
        {
            //
        }

        // Tetris frame

        final Tetris tetris = new Tetris ();
        tetris.setUseInternalHotkeys ( false );

        final WebInternalFrame tetrisFrame =
                new WebInternalFrame ( "Tetris frame", false, true, false, true )
                {
                    public void setVisible ( boolean aFlag )
                    {
                        if ( !aFlag )
                        {
                            tetris.pauseGame ();
                        }
                        super.setVisible ( aFlag );
                    }
                };
        tetrisFrame.setFrameIcon ( game );
        tetrisFrame.add ( tetris );
        desktopPane.add ( tetrisFrame );

        JMenuBar tetrisMenu = new JMenuBar ();
        tetrisMenu.add ( new JMenu ( "Game" )
        {
            {
                add ( new JMenuItem ( "New game", newGame )
                {
                    {
                        setAccelerator ( KeyStroke.getKeyStroke ( KeyEvent.VK_F2, 0 ) );
                        addActionListener ( new ActionListener ()
                        {
                            public void actionPerformed ( ActionEvent e )
                            {
                                tetris.newGame ();
                            }
                        } );
                    }
                } );
                add ( new JMenuItem ( "Unpause game", unpauseGame )
                {
                    {
                        tetris.addTetrisListener ( new TetrisListener ()
                        {
                            public void newGameStarted ()
                            {
                                setEnabled ( true );
                                setIcon ( pauseGame );
                                setText ( "Pause game" );
                            }

                            public void gamePaused ()
                            {
                                setIcon ( unpauseGame );
                                setText ( "Unpause game" );
                            }

                            public void gameUnpaused ()
                            {
                                setIcon ( pauseGame );
                                setText ( "Pause game" );
                            }

                            public void gameOver ()
                            {
                                setEnabled ( false );
                                setIcon ( pauseGame );
                                setText ( "Pause game" );
                            }
                        } );
                        setAccelerator ( KeyStroke.getKeyStroke ( KeyEvent.VK_P, 0 ) );
                        addActionListener ( new ActionListener ()
                        {
                            public void actionPerformed ( ActionEvent e )
                            {
                                if ( tetris.isPaused () )
                                {
                                    tetris.unpauseGame ();
                                }
                                else
                                {
                                    tetris.pauseGame ();
                                }
                            }
                        } );
                    }
                } );
                addSeparator ();
                add ( new JMenuItem ( "Exit", exitGame )
                {
                    {
                        setAccelerator (
                                KeyStroke.getKeyStroke ( KeyEvent.VK_F4, KeyEvent.SHIFT_MASK ) );
                        addActionListener ( new ActionListener ()
                        {
                            public void actionPerformed ( ActionEvent e )
                            {
                                try
                                {
                                    tetris.pauseGame ();
                                    tetrisFrame.setClosed ( true );
                                }
                                catch ( PropertyVetoException e1 )
                                {
                                    //
                                }
                            }
                        } );
                    }
                } );
            }
        } );
        tetrisMenu.add ( new JMenu ( "About" ) );
        tetrisFrame.setJMenuBar ( tetrisMenu );

        // Tetris frame icon

        final WebButton tetrisFrameIcon = new WebButton ( "Tetris", tetris32 );
        tetrisFrameIcon.setRolloverDecoratedOnly ( true );
        tetrisFrameIcon.setHorizontalTextPosition ( WebButton.CENTER );
        tetrisFrameIcon.setVerticalTextPosition ( WebButton.BOTTOM );
        tetrisFrameIcon.addActionListener ( new ActionListener ()
        {
            private boolean loading = false;
            private boolean firstLoad = true;

            public void actionPerformed ( ActionEvent e )
            {
                if ( loading ||
                        tetrisFrameIcon.getClientProperty ( ObjectMoveAdapter.DRAGGED_MARK ) !=
                                null )
                {
                    return;
                }

                tetrisFrameIcon.setIcon ( loader );
                loading = true;

                new Thread ( new Runnable ()
                {

                    public void run ()
                    {
                        if ( firstLoad )
                        {
                            firstLoad = false;
                            ThreadUtils.sleep ( 1000 );
                        }

                        if ( tetrisFrame.isClosed () )
                        {
                            try
                            {
                                if ( tetrisFrame.getParent () == null )
                                {
                                    desktopPane.add ( tetrisFrame );
                                }
                                tetrisFrame.setClosed ( false );
                                tetrisFrame.setVisible ( true );
                                tetrisFrame.setIcon ( false );
                            }
                            catch ( PropertyVetoException e1 )
                            {
                                //
                            }
                        }
                        else
                        {
                            try
                            {
                                tetrisFrame.setIcon ( !tetrisFrame.isIcon () );
                            }
                            catch ( PropertyVetoException e1 )
                            {
                                //
                            }
                        }
                        SwingUtilities.invokeLater ( new Runnable ()
                        {
                            public void run ()
                            {
                                tetrisFrameIcon.setIcon ( tetris32 );
                                loading = false;
                            }
                        } );
                    }
                } ).start ();
            }
        } );
        ObjectMoveAdapter ma2 = new ObjectMoveAdapter ();
        tetrisFrameIcon.addMouseListener ( ma2 );
        tetrisFrameIcon.addMouseMotionListener ( ma2 );
        tetrisFrameIcon.setBounds ( 25, 25, 100, 75 );
        desktopPane.add ( tetrisFrameIcon );

        tetrisFrame.pack ();
        tetrisFrame.setLocation ( 25 + 100 + 25, 25 );
        tetrisFrame.setVisible ( true );
        try
        {
            tetrisFrame.setClosed ( true );
        }
        catch ( PropertyVetoException e )
        {
            //
        }

        return desktopPane;
    }
}