/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.examples.groups.field;

import com.alee.examples.DefaultExample;
import com.alee.examples.WebLookAndFeelDemo;
import com.alee.extended.panel.GroupPanel;
import com.alee.laf.checkbox.WebCheckBox;
import com.alee.laf.text.WebPasswordField;
import com.alee.laf.text.WebTextField;
import com.alee.managers.tooltip.TooltipManager;
import com.alee.managers.tooltip.TooltipWay;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * User: mgarin Date: 23.01.12 Time: 17:22
 */

public class FieldComponentsExample extends DefaultExample
{
    private static final ImageIcon search =
            new ImageIcon ( FieldComponentsExample.class.getResource ( "icons/search.png" ) );
    private static final ImageIcon key =
            new ImageIcon ( FieldComponentsExample.class.getResource ( "icons/key.png" ) );
    private static final ImageIcon exclamation =
            new ImageIcon ( FieldComponentsExample.class.getResource ( "icons/exclamation.png" ) );

    public String getTitle ()
    {
        return "Field components";
    }

    public String getDescription ()
    {
        return "Fields with leading/trailing components";
    }

    public Component getPreview ( WebLookAndFeelDemo owner )
    {
        final WebTextField leading = new WebTextField ( "Leading" );
        leading.setLeadingComponent ( new WebCheckBox ()
        {
            {
                setCursor ( Cursor.getDefaultCursor () );
                setSelected ( true );
                addActionListener ( new ActionListener ()
                {
                    public void actionPerformed ( ActionEvent e )
                    {
                        leading.setEnabled ( isSelected () );
                    }
                } );
            }
        } );

        WebTextField trailing = new WebTextField ( "Trailing" );
        trailing.setTrailingComponent ( new JLabel ( search ) );

        WebPasswordField leading2 = new WebPasswordField ( "Leading" );
        leading2.setLeadingComponent ( new JLabel ( key ) );

        WebPasswordField trailing2 = new WebPasswordField ( "Trailing" );
        trailing2.setTrailingComponent ( new JLabel ( key ) );

        WebTextField both = new WebTextField ( "Both", 10 );
        both.setHorizontalAlignment ( WebTextField.CENTER );
        both.setLeadingComponent ( new JLabel ( exclamation )
        {
            {
                setCursor ( Cursor.getDefaultCursor () );
                TooltipManager.setTooltip ( this, "Some left message", TooltipWay.left, 0 );
            }
        } );
        both.setTrailingComponent ( new JLabel ( exclamation )
        {
            {
                setCursor ( Cursor.getDefaultCursor () );
                TooltipManager.setTooltip ( this, "Some right message", TooltipWay.right, 0 );
            }
        } );

        return new GroupPanel ( false, new GroupPanel ( leading ), new GroupPanel ( trailing ),
                new GroupPanel ( leading2 ), new GroupPanel ( trailing2 ),
                new GroupPanel ( both ) );
    }
}