/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.examples.groups.menubar;

import com.alee.examples.DefaultExampleGroup;
import com.alee.examples.Example;
import com.alee.laf.menu.WebMenu;
import com.alee.laf.menu.WebMenuBar;
import com.alee.laf.menu.WebMenuItem;
import com.alee.managers.hotkey.Hotkey;
import com.alee.managers.tooltip.TooltipManager;
import com.alee.managers.tooltip.TooltipWay;

import javax.swing.*;
import java.util.ArrayList;
import java.util.List;

/**
 * User: mgarin Date: 24.01.12 Time: 14:48
 */

public class MenuBarsGroup extends DefaultExampleGroup
{
    private static final ImageIcon menubarFile =
            new ImageIcon ( MenuBarsGroup.class.getResource ( "icons/menubar/file.png" ) );
    private static final ImageIcon menubarFileMedia =
            new ImageIcon ( MenuBarsGroup.class.getResource ( "icons/menubar/media.png" ) );
    private static final ImageIcon menubarFileImage =
            new ImageIcon ( MenuBarsGroup.class.getResource ( "icons/menubar/file_image.png" ) );
    private static final ImageIcon menubarFileMusic =
            new ImageIcon ( MenuBarsGroup.class.getResource ( "icons/menubar/file_music.png" ) );
    private static final ImageIcon menubarFileVideo =
            new ImageIcon ( MenuBarsGroup.class.getResource ( "icons/menubar/file_video.png" ) );
    private static final ImageIcon menubarFileArchive =
            new ImageIcon ( MenuBarsGroup.class.getResource ( "icons/menubar/file_archive.png" ) );
    private static final ImageIcon menubarFileDoc =
            new ImageIcon ( MenuBarsGroup.class.getResource ( "icons/menubar/file_doc.png" ) );
    private static final ImageIcon menubarFileXls =
            new ImageIcon ( MenuBarsGroup.class.getResource ( "icons/menubar/file_xls.png" ) );
    private static final ImageIcon menubarFilePpt =
            new ImageIcon ( MenuBarsGroup.class.getResource ( "icons/menubar/file_ppt.png" ) );
    private static final ImageIcon menubarFileExit =
            new ImageIcon ( MenuBarsGroup.class.getResource ( "icons/menubar/file_exit.png" ) );
    private static final ImageIcon menubarEdit =
            new ImageIcon ( MenuBarsGroup.class.getResource ( "icons/menubar/edit.png" ) );
    private static final ImageIcon menubarEditCut =
            new ImageIcon ( MenuBarsGroup.class.getResource ( "icons/menubar/edit_cut.png" ) );
    private static final ImageIcon menubarEditCopy =
            new ImageIcon ( MenuBarsGroup.class.getResource ( "icons/menubar/edit_copy.png" ) );
    private static final ImageIcon menubarEditPaste =
            new ImageIcon ( MenuBarsGroup.class.getResource ( "icons/menubar/edit_paste.png" ) );
    private static final ImageIcon menubarTooltip =
            new ImageIcon ( MenuBarsGroup.class.getResource ( "icons/menubar/tooltip.png" ) );

    public Icon getGroupIcon ()
    {
        return new ImageIcon ( MenuBarsGroup.class.getResource ( "icons/menubar.png" ) );
    }

    public String getGroupName ()
    {
        return "Menubars";
    }

    public String getGroupDescription ()
    {
        return "Various examples of menubars usage";
    }

    public List<Example> getGroupExamples ()
    {
        List<Example> examples = new ArrayList<Example> ();
        examples.add ( new SimpleMenuBarExample () );
        examples.add ( new StandaloneMenuBarExample () );
        examples.add ( new UndecoratedMenuBarExample () );
        return examples;
    }

    public static void setupMenuBar ( WebMenuBar menuBar )
    {
        menuBar.add ( new WebMenu ( "File", menubarFile )
        {
            {
                add ( new WebMenu ( "New media file", menubarFileMedia )
                {
                    {
                        add ( new WebMenuItem ( "New image", menubarFileImage )
                        {
                            {
                                setHotkey ( Hotkey.CTRL_N );
                            }
                        } );
                        add ( new WebMenuItem ( "New music", menubarFileMusic ) );
                        add ( new WebMenuItem ( "New video", menubarFileVideo ) );
                        add ( new WebMenuItem ( "New archive", menubarFileArchive ) );
                    }
                } );
                addSeparator ();
                add ( new WebMenuItem ( "New text document", menubarFileDoc ) );
                add ( new WebMenuItem ( "New excel table", menubarFileXls ) );
                add ( new WebMenuItem ( "New presentation", menubarFilePpt ) );
                addSeparator ();
                add ( new WebMenuItem ( "Exit", menubarFileExit )
                {
                    {
                        setHotkey ( Hotkey.ALT_F4 );
                    }
                } );
            }
        } );
        menuBar.add ( new WebMenu ( "Edit", menubarEdit )
        {
            {
                add ( new WebMenuItem ( "Cut", menubarEditCut )
                {
                    {
                        setHotkey ( Hotkey.CTRL_X );
                    }
                } );
                add ( new WebMenuItem ( "Copy", menubarEditCopy )
                {
                    {
                        setHotkey ( Hotkey.CTRL_C );
                    }
                } );
                add ( new WebMenuItem ( "Paste", menubarEditPaste )
                {
                    {
                        setHotkey ( Hotkey.CTRL_V );
                    }
                } );
            }
        } );
        menuBar.add ( new WebMenu ( "Menu tooltip", menubarTooltip )
        {
            {
                TooltipManager.setTooltip ( this, "Menu tooltip" );
                add ( new WebMenuItem ( "Menu item tooltip", menubarTooltip )
                {
                    {
                        TooltipManager.setTooltip ( this, "Menu item tooltip", TooltipWay.right );
                    }
                } );
            }
        } );
    }
}