/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.examples.groups.panel;

import com.alee.examples.DefaultExample;
import com.alee.examples.WebLookAndFeelDemo;
import com.alee.extended.label.WebVerticalLabel;
import com.alee.extended.panel.GroupPanel;
import com.alee.laf.label.WebLabel;
import com.alee.laf.panel.WebPanel;
import info.clearthought.layout.TableLayout;

import java.awt.*;

/**
 * User: mgarin Date: 15.02.12 Time: 15:33
 */

public class PartialPanelsExample extends DefaultExample
{
    public String getTitle ()
    {
        return "Partially decorated panels";
    }

    public String getDescription ()
    {
        return "Web-styled panels with missing sides";
    }

    public Component getPreview ( WebLookAndFeelDemo owner )
    {
        return new GroupPanel ( createFirstPanel (), createSecondPanel () );
    }

    private WebPanel createFirstPanel ()
    {
        WebPanel panel = new WebPanel ();
        panel.setUndecorated ( false );
        panel.setLayout ( new BorderLayout () );
        panel.setWebColored ( false );

        final Insets margin = new Insets ( 3, 3, 3, 3 );

        panel.add ( new WebPanel ()
        {
            {
                setLayout ( new BorderLayout () );
                setUndecorated ( false );
                setDrawTop ( false );
                setDrawLeft ( false );
                setDrawRight ( false );
                setMargin ( margin );
                add ( new WebLabel ( "North panel", WebLabel.CENTER ) );
            }
        }, BorderLayout.NORTH );

        panel.add ( new WebPanel ()
        {
            {
                setLayout ( new BorderLayout () );
                setUndecorated ( false );
                setDrawBottom ( false );
                setDrawLeft ( false );
                setDrawRight ( false );
                setMargin ( margin );
                add ( new WebLabel ( "South panel", WebLabel.CENTER ) );
            }
        }, BorderLayout.SOUTH );

        panel.add ( new WebPanel ()
        {
            {
                setLayout ( new BorderLayout () );
                setUndecorated ( false );
                setDrawLeft ( false );
                setMargin ( margin );
                add ( new WebVerticalLabel ( "West panel", WebLabel.CENTER, false ) );
            }
        }, BorderLayout.WEST );

        panel.add ( new WebPanel ()
        {
            {
                setLayout ( new BorderLayout () );
                setUndecorated ( false );
                setDrawRight ( false );
                setMargin ( margin );
                add ( new WebVerticalLabel ( "East panel", WebLabel.CENTER, true ) );
            }
        }, BorderLayout.EAST );

        panel.add ( new WebPanel ()
        {
            {
                setLayout ( new BorderLayout () );
                setUndecorated ( false );
                setMargin ( margin );
                add ( new WebLabel ( "Center panel", WebLabel.CENTER ) );
            }
        }, BorderLayout.CENTER );

        return panel;
    }

    private WebPanel createSecondPanel ()
    {
        WebPanel panel = new WebPanel ();
        panel.setUndecorated ( false );
        panel.setLayout ( new TableLayout (
                new double[][]{ { TableLayout.PREFERRED, TableLayout.FILL, TableLayout.PREFERRED },
                        { TableLayout.PREFERRED, TableLayout.FILL, TableLayout.PREFERRED } } ) );
        panel.setWebColored ( false );

        final Insets margin = new Insets ( 3, 3, 3, 3 );

        panel.add ( new WebPanel ()
        {
            {
                setLayout ( new BorderLayout () );
                setUndecorated ( false );
                setDrawTop ( false );
                setDrawLeft ( false );
                setMargin ( margin );
                add ( new WebLabel ( "North panel", WebLabel.CENTER ) );
            }
        }, "0,0,1,0" );

        panel.add ( new WebPanel ()
        {
            {
                setLayout ( new BorderLayout () );
                setUndecorated ( false );
                setDrawBottom ( false );
                setDrawRight ( false );
                setMargin ( margin );
                add ( new WebLabel ( "South panel", WebLabel.CENTER ) );
            }
        }, "1,2,2,2" );

        panel.add ( new WebPanel ()
        {
            {
                setLayout ( new BorderLayout () );
                setUndecorated ( false );
                setDrawLeft ( false );
                setDrawBottom ( false );
                setMargin ( margin );
                add ( new WebVerticalLabel ( "West panel", WebLabel.CENTER, false ) );
            }
        }, "0,1,0,2" );

        panel.add ( new WebPanel ()
        {
            {
                setLayout ( new BorderLayout () );
                setUndecorated ( false );
                setDrawRight ( false );
                setDrawTop ( false );
                setMargin ( margin );
                add ( new WebVerticalLabel ( "East panel", WebLabel.CENTER, true ) );
            }
        }, "2,0,2,1" );

        panel.add ( new WebPanel ()
        {
            {
                setLayout ( new BorderLayout () );
                setUndecorated ( false );
                setMargin ( margin );
                add ( new WebLabel ( "Center panel", WebLabel.CENTER ) );
            }
        }, "1,1" );

        return panel;
    }
}