/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.examples.groups.progress;

import com.alee.examples.DefaultExample;
import com.alee.examples.FeatureState;
import com.alee.examples.WebLookAndFeelDemo;
import com.alee.extended.panel.GroupPanel;
import com.alee.extended.window.WebProgressDialog;
import com.alee.laf.button.WebButton;
import com.alee.utils.ThreadUtils;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * User: mgarin Date: 14.02.12 Time: 12:44
 */

public class WebProgressDialogExample extends DefaultExample
{
    public String getTitle ()
    {
        return "Progress dialog";
    }

    public String getDescription ()
    {
        return "WebProgressDialog usage example";
    }

    public FeatureState getFeatureState ()
    {
        return FeatureState.alpha;
    }

    public Component getPreview ( WebLookAndFeelDemo owner )
    {
        final WebButton showModalLoad = new WebButton ( "Show modal load dialog" );
        showModalLoad.addActionListener ( new ActionListener ()
        {
            public void actionPerformed ( ActionEvent e )
            {
                // Load dialog
                Window wa = SwingUtilities.getWindowAncestor ( showModalLoad );
                final WebProgressDialog progress = new WebProgressDialog ( wa, "Some progress" );
                progress.setTitleText ( "Loading something..." );

                // Starting updater thread
                new Thread ( new Runnable ()
                {
                    public void run ()
                    {
                        for ( int i = 0; i <= 100; i++ )
                        {
                            ThreadUtils.sleep ( 50 );
                            progress.setProgress ( i );
                            if ( i == 25 )
                            {
                                progress.setTitleText ( "1/4 done" );
                            }
                            else if ( i == 50 )
                            {
                                progress.setTitleText ( "Half done" );
                            }
                            else if ( i == 75 )
                            {
                                progress.setTitleText ( "3/4 done" );
                            }
                        }
                        progress.setTitleText ( "Done! Closing in 3..." );
                        ThreadUtils.sleep ( 1000 );
                        progress.setTitleText ( "Done! Closing in 2..." );
                        ThreadUtils.sleep ( 1000 );
                        progress.setTitleText ( "Done! Closing in 1..." );
                        ThreadUtils.sleep ( 1000 );
                        progress.setVisible ( false );
                    }
                } ).start ();

                progress.setModal ( true );
                progress.setVisible ( true );
            }
        } );
        return new GroupPanel ( showModalLoad );
    }
}