/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.examples.groups.progress;

import com.alee.examples.DefaultExample;
import com.alee.examples.FeatureState;
import com.alee.examples.WebLookAndFeelDemo;
import com.alee.extended.image.WebProgressPainter;
import com.alee.extended.panel.GroupPanel;
import com.alee.laf.button.WebButton;
import com.alee.laf.scroll.WebScrollPane;
import com.alee.laf.text.WebTextArea;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

/**
 * User: mgarin Date: 21.02.12 Time: 16:23
 */

public class WebProgressPainterExample extends DefaultExample
{
    public String getTitle ()
    {
        return "Progress painter";
    }

    public String getDescription ()
    {
        return "WebProgressPainter example";
    }

    public FeatureState getFeatureState ()
    {
        return FeatureState.beta;
    }

    public Component getPreview ( WebLookAndFeelDemo owner )
    {
        Class<WebProgressPainterExample> nearClass = WebProgressPainterExample.class;
        final ImageIcon start = new ImageIcon ( nearClass.getResource ( "icons/start.png" ) );
        final ImageIcon stop = new ImageIcon ( nearClass.getResource ( "icons/stop.png" ) );

        final WebProgressPainter progressPainter1 = new WebProgressPainter ();
        WebButton button = new WebButton ( "Click to start", start )
        {
            {
                addActionListener ( new ActionListener ()
                {
                    public void actionPerformed ( ActionEvent e )
                    {
                        boolean showLoad = !progressPainter1.isShowLoad ();
                        progressPainter1.setShowLoad ( showLoad );
                        setText ( showLoad ? "Click to stop" : "Click to start" );
                        setIcon ( showLoad ? stop : start );
                    }
                } );
            }
        };
        progressPainter1.setComponent ( button );


        final WebProgressPainter progressPainter2 = new WebProgressPainter ();
        progressPainter2.setProgressWidth ( 35 );

        WebTextArea textArea = new WebTextArea ();
        textArea.setText ( "Click inside area to start\n" + "Click again to stop" );
        textArea.addMouseListener ( new MouseAdapter ()
        {
            public void mousePressed ( MouseEvent e )
            {
                progressPainter2.setShowLoad ( !progressPainter2.isShowLoad () );
            }
        } );

        WebScrollPane scrollPane = new WebScrollPane ( textArea );
        scrollPane.setPreferredSize ( new Dimension ( 250, 100 ) );
        progressPainter2.setComponent ( scrollPane );


        return new GroupPanel ( 4, false, progressPainter1, progressPainter2 );
    }
}