/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.examples.groups.progressbar;

import com.alee.examples.DefaultExample;
import com.alee.examples.WebLookAndFeelDemo;
import com.alee.extended.panel.GroupPanel;
import com.alee.laf.StyleConstants;
import com.alee.laf.progressbar.WebProgressBar;
import com.alee.utils.swing.Timer;

import javax.swing.event.AncestorEvent;
import javax.swing.event.AncestorListener;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * User: mgarin Date: 16.02.12 Time: 15:10
 */

public class HorizontalProgressBarsExample extends DefaultExample
{
    public String getTitle ()
    {
        return "Horizontal progress bars";
    }

    public String getDescription ()
    {
        return "Horizontal Web-styled progress bars";
    }

    public Component getPreview ( WebLookAndFeelDemo owner )
    {
        final WebProgressBar progressBar1 = new WebProgressBar ();
        progressBar1.setIndeterminate ( false );
        progressBar1.setMinimum ( 0 );
        progressBar1.setMaximum ( 100 );
        progressBar1.setValue ( 0 );
        progressBar1.setStringPainted ( true );

        final WebProgressBar progressBar2 = new WebProgressBar ();
        progressBar2.setIndeterminate ( false );
        progressBar2.setMinimum ( 0 );
        progressBar2.setMaximum ( 100 );
        progressBar2.setValue ( 0 );
        progressBar2.setStringPainted ( false );

        WebProgressBar progressBar3 = new WebProgressBar ();
        progressBar3.setIndeterminate ( true );
        progressBar3.setStringPainted ( false );
        progressBar3.setStringPainted ( true );
        progressBar3.setString ( "Please wait..." );

        WebProgressBar progressBar4 = new WebProgressBar ();
        progressBar4.setIndeterminate ( true );
        progressBar4.setStringPainted ( false );

        WebProgressBar progressBar5 = new WebProgressBar ();
        progressBar5.setIndeterminate ( false );
        progressBar5.setMinimum ( 0 );
        progressBar5.setMaximum ( 100 );
        progressBar5.setValue ( 75 );
        progressBar5.setStringPainted ( true );
        progressBar5.setEnabled ( false );

        WebProgressBar progressBar6 = new WebProgressBar ();
        progressBar6.setIndeterminate ( true );
        progressBar6.setEnabled ( false );

        GroupPanel view = new GroupPanel ( 4, false, new GroupPanel ( progressBar1 ),
                new GroupPanel ( progressBar2 ), new GroupPanel ( progressBar3 ),
                new GroupPanel ( progressBar4 ), new GroupPanel ( progressBar5 ),
                new GroupPanel ( progressBar6 ) );

        // Fake progress timer
        final Timer timer = new Timer ( 1000 / StyleConstants.animationFps, new ActionListener ()
        {
            private boolean increasing = true;

            public void actionPerformed ( ActionEvent e )
            {
                int value = progressBar1.getValue ();
                if ( increasing )
                {
                    progressBar1.setValue ( value + 1 );
                    progressBar2.setValue ( value + 1 );
                    if ( value + 1 == 100 )
                    {
                        increasing = false;
                    }
                }
                else
                {
                    progressBar1.setValue ( value - 1 );
                    progressBar2.setValue ( value - 1 );
                    if ( value - 1 == 0 )
                    {
                        increasing = true;
                    }
                }
            }
        } );

        // To avoid useless timer runtime
        view.addAncestorListener ( new AncestorListener ()
        {
            public void ancestorAdded ( AncestorEvent event )
            {
                timer.start ();
            }

            public void ancestorRemoved ( AncestorEvent event )
            {
                timer.stop ();
            }

            public void ancestorMoved ( AncestorEvent event )
            {
                //
            }
        } );

        return view;
    }
}