/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.examples.groups.tabbedpane;

import com.alee.examples.DefaultExample;
import com.alee.examples.WebLookAndFeelDemo;
import com.alee.extended.panel.GroupPanel;
import com.alee.laf.label.WebLabel;
import com.alee.laf.tabbedpane.WebTabbedPane;

import javax.swing.*;
import java.awt.*;

/**
 * User: mgarin Date: 17.02.12 Time: 14:33
 */

public class SimpleTabbedPanesExample extends DefaultExample
{
    public String getTitle ()
    {
        return "Simple tabbed panes";
    }

    public String getDescription ()
    {
        return "Simple Web-styled tabbed panes";
    }

    public Component getPreview ( WebLookAndFeelDemo owner )
    {
        WebTabbedPane tabbedPane1 = new WebTabbedPane ()
        {
            {
                setPreferredSize ( new Dimension ( 150, 100 ) );
                setTabPlacement ( WebTabbedPane.TOP );
                setupTabbedPane ( this );
            }
        };

        WebTabbedPane tabbedPane2 = new WebTabbedPane ()
        {
            {
                setPreferredSize ( new Dimension ( 150, 100 ) );
                setTabPlacement ( WebTabbedPane.BOTTOM );
                setupTabbedPane ( this );
            }
        };

        WebTabbedPane tabbedPane3 = new WebTabbedPane ()
        {
            {
                setPreferredSize ( new Dimension ( 150, 120 ) );
                setTabPlacement ( WebTabbedPane.LEFT );
                setupTabbedPane ( this );
            }
        };

        WebTabbedPane tabbedPane4 = new WebTabbedPane ()
        {
            {
                setPreferredSize ( new Dimension ( 150, 120 ) );
                setTabPlacement ( WebTabbedPane.RIGHT );
                setupTabbedPane ( this );
            }
        };

        return new GroupPanel ( 4, false, new GroupPanel ( 4, tabbedPane1, tabbedPane2 ),
                new GroupPanel ( 4, tabbedPane3, tabbedPane4 ) );
    }

    private static void setupTabbedPane ( JTabbedPane tabbedPane )
    {
        tabbedPane.addTab ( "Normal 1", new WebLabel () );

        tabbedPane.addTab ( "Disabled 2", new WebLabel () );
        tabbedPane.setEnabledAt ( 1, false );

        tabbedPane.addTab ( "Selected 3", new WebLabel () );
        tabbedPane.setSelectedIndex ( 2 );

        tabbedPane.addTab ( "Colored 4", new WebLabel () );
        tabbedPane.setForegroundAt ( 3, new Color ( 60, 66, 149 ) );
        tabbedPane.setBackgroundAt ( 3, new Color ( 235, 235, 235 ) );
    }
}