/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.examples.groups.tooltip;

import com.alee.examples.DefaultExample;
import com.alee.examples.WebLookAndFeelDemo;
import com.alee.extended.panel.GroupPanel;
import com.alee.laf.button.WebButton;
import com.alee.laf.label.WebLabel;
import com.alee.laf.progressbar.WebProgressBar;
import com.alee.managers.hotkey.Hotkey;
import com.alee.managers.hotkey.HotkeyManager;
import com.alee.managers.tooltip.TooltipManager;
import com.alee.managers.tooltip.TooltipWay;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * User: mgarin Date: 17.02.12 Time: 13:57
 */

public class CustomTooltipsExample extends DefaultExample
{
    public String getTitle ()
    {
        return "Custom tooltips";
    }

    public String getDescription ()
    {
        return "Custom Web-styled tooltips";
    }

    public Component getPreview ( final WebLookAndFeelDemo owner )
    {
        // This is custom tooltip set by TooltipManager
        WebButton tip1 = new WebButton ( "Custom shaped tooltip" );
        TooltipManager.setTooltip ( tip1, TooltipManager.getDefaultDelay () + " ms delay",
                TooltipWay.right );

        // You can easily remove tooltip delay for certain component
        WebButton tip2 = new WebButton ( "Zero delay tooltip" );
        TooltipManager.setTooltip ( tip2, "You see this right away", TooltipWay.right, 0 );

        // Double tooltip for single component
        WebButton tip3 = new WebButton ( "Double tooltip" );
        TooltipManager.setTooltip ( tip3, "Left side tooltip", TooltipWay.left, 0 );
        TooltipManager.setTooltip ( tip3, "Right side tooltip", TooltipWay.right, 0 );

        // Hotkey and Tooltip managers integration
        WebButton tip4 = new WebButton ( "Hotkey in tooltip" );
        HotkeyManager.registerHotkey ( owner, tip4, Hotkey.CTRL_E, TooltipWay.right );
        TooltipManager.setTooltip ( tip4, "Hotkey tip is added to tooltip", TooltipWay.right, 0 );
        final WebLabel clickCount = new WebLabel ();
        tip4.addActionListener ( new ActionListener ()
        {
            private int count = 0;

            public void actionPerformed ( ActionEvent e )
            {
                count++;
                clickCount.setText ( "Pressed " + count + ( count == 1 ? " time" : " times" ) );
            }
        } );

        // One-time tooltip that could be shown once anywhere you want
        final WebButton tip5 = new WebButton ( "On-click tooltip" );
        tip5.addActionListener ( new ActionListener ()
        {
            public void actionPerformed ( ActionEvent e )
            {
                // Show tooltip where we want it to be
                Point p = new Point ( tip5.getWidth () - tip5.getShadeWidth (),
                        tip5.getHeight () / 2 );
                TooltipManager.showOneTimeTooltip ( tip5, p, "<html><center>One-time tooltip<br>" +
                        "with <b>HTML</b> content</center></html>", TooltipWay.right );
            }
        } );

        // You can show any kind of content inside the tooltip
        WebButton tip6 = new WebButton ( "Custom content tooltip" );
        final WebProgressBar progressBar = new WebProgressBar ();
        progressBar.setIndeterminate ( true );
        TooltipManager.setTooltip ( tip6, progressBar, TooltipWay.down, 0 )
                .setLeftRightSpacing ( 0 );

        return new GroupPanel ( 4, false, new GroupPanel ( tip1 ), new GroupPanel ( tip2 ),
                new GroupPanel ( tip3 ), new GroupPanel ( 4, tip4, clickCount ),
                new GroupPanel ( tip5 ), new GroupPanel ( tip6 ) );
    }
}