/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.examples.groups.transition;

import com.alee.examples.DefaultExample;
import com.alee.examples.FeatureState;
import com.alee.examples.WebLookAndFeelDemo;
import com.alee.examples.groups.gallery.GalleryExample;
import com.alee.extended.image.WebImageTransition;
import com.alee.extended.image.transition.CurtainType;
import com.alee.extended.image.transition.Direction;
import com.alee.extended.image.transition.SlideType;
import com.alee.extended.image.transition.TransitionEffect;
import com.alee.extended.panel.TransitionPanel;
import com.alee.laf.label.WebLabel;
import com.alee.laf.panel.WebPanel;
import info.clearthought.layout.TableLayout;

import javax.swing.*;
import java.awt.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.List;

/**
 * User: mgarin Date: 17.02.12 Time: 18:23
 */

public class ImagesTransitionExample extends DefaultExample
{
    public String getTitle ()
    {
        return "Images transition";
    }

    public String getDescription ()
    {
        return "Images transition effects";
    }

    public FeatureState getFeatureState ()
    {
        return FeatureState.beta;
    }

    public Component getPreview ( WebLookAndFeelDemo owner )
    {
        WebPanel effectsPanel = new WebPanel ();
        effectsPanel.setOpaque ( false );
        TableLayout effectspaneLayout = new TableLayout ( new double[][]{
                { 20, TableLayout.FILL, TableLayout.PREFERRED, TableLayout.PREFERRED,
                        TableLayout.FILL, 20 },
                { TableLayout.FILL, 20, TableLayout.PREFERRED, TableLayout.PREFERRED, 20,
                        TableLayout.FILL } } );
        effectspaneLayout.setHGap ( 4 );
        effectspaneLayout.setVGap ( 4 );
        effectsPanel.setLayout ( effectspaneLayout );

        effectsPanel.add ( new WebLabel ( "Click on the image to see various transitions",
                WebLabel.CENTER ), "2,2" );

        // Slide components (any component or group of components can be used)
        final List<JComponent> slides = new ArrayList<JComponent> ();
        slides.add ( new JLabel ( GalleryExample.largeImage1 ) );
        slides.add ( new JLabel ( GalleryExample.largeImage2 ) );
        slides.add ( new JLabel ( GalleryExample.largeImage3 ) );
        slides.add ( new JLabel ( GalleryExample.largeImage4 ) );
        slides.add ( new JLabel ( GalleryExample.largeImage5 ) );

        // Transition panel and its settings
        final TransitionPanel imageTransitionPanel = new TransitionPanel ( slides.get ( 0 ) );
        imageTransitionPanel.setTransitionEffect ( TransitionEffect.random );
        imageTransitionPanel.put ( WebImageTransition.DIRECTION, Direction.random );
        imageTransitionPanel.put ( WebImageTransition.BLOCK_TRANSITION_TRANSPARENCY, true );
        imageTransitionPanel.put ( WebImageTransition.CURTAIN_TYPE, CurtainType.random );
        imageTransitionPanel.put ( WebImageTransition.SLIDE_TYPE, SlideType.random );
        effectsPanel.add ( new WebPanel ( true, imageTransitionPanel ), "2,3" );

        // Switching listeners
        for ( int i = 0; i < slides.size (); i++ )
        {
            final int index = i;
            slides.get ( i ).addMouseListener ( new MouseAdapter ()
            {
                public void mousePressed ( MouseEvent e )
                {
                    imageTransitionPanel.switchContent (
                            index < slides.size () - 1 ? slides.get ( index + 1 ) :
                                    slides.get ( 0 ) );
                }
            } );
        }

        return effectsPanel;
    }
}