/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.extended.background;

import javax.swing.*;
import java.awt.*;

/**
 * User: mgarin Date: 14.12.11 Time: 15:10
 * <p/>
 * This interface provides basic methods for component background (or component parts) painting. It
 * is supported in most of the Web-components an Web-UIs so that you can quickly and easily change
 * any component view. It also provides preferred size and margin that will also be taken into
 * account when component sizes and its content margins are calcualted.
 */

public interface BackgroundPainter
{
    /**
     * Returned value will affect component's (which uses this background) opacity, which means that
     * if "true" is returned - the component will be also made opaque when this BackgroundPainter is
     * set for painting its background duty
     */
    public boolean isOpaque ();

    /**
     * Preferred (more like minimum) background size in which background can be properly painted
     */
    public Dimension getBackgroundPreferredSize ( JComponent c );

    /**
     * Preferred background margin which could be used by components to set their borders/margins
     */
    public Insets getBackgroundMargin ( JComponent c );

    /**
     * Main background painting method
     * <p/>
     * Graphics and JComponent are transferred directly from component UI paint method
     * <p/>
     * Rectangle could be sometimes specified by the component UI, but in most cases it fits full
     * component bounds (0,0,c.getWidth(),c.geHeight())
     */
    public void paintBackground ( Graphics g, Rectangle bounds, JComponent c );
}
