/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.extended.breadcrumb;

import com.alee.extended.background.DefaultBackgroundPainter;
import com.alee.laf.StyleConstants;
import com.alee.utils.LafUtils;

import javax.swing.*;
import java.awt.*;
import java.awt.geom.GeneralPath;

/**
 * User: mgarin Date: 01.03.12 Time: 19:57
 */

public class BreadcrumbElementPainter extends DefaultBackgroundPainter
{
    private int overlap = BreadcrumbStyle.overlap;

    private int shadeWidth = BreadcrumbStyle.shadeWidth;
    private Color borderColor = BreadcrumbStyle.borderColor;
    private Color disabledBorderColor = BreadcrumbStyle.disabledBorderColor;

    private Color bgTop = BreadcrumbStyle.bgTop;
    private Color bgBottom = BreadcrumbStyle.bgBottom;
    private Color selectedBgColor = BreadcrumbStyle.selectedBgColor;

    private BreadcrumbElementType type;

    public BreadcrumbElementPainter ( BreadcrumbElementType type )
    {
        super ();
        this.type = type;
    }

    public int getOverlap ()
    {
        return overlap;
    }

    public void setOverlap ( int overlap )
    {
        this.overlap = overlap;
    }

    public Insets getBackgroundMargin ( JComponent c )
    {
        return new Insets ( 0, !type.equals ( BreadcrumbElementType.start ) ? overlap : 0, 0,
                !type.equals ( BreadcrumbElementType.end ) ? overlap + shadeWidth : 0 );
    }

    public void paintBackground ( Graphics g, Rectangle bounds, JComponent c )
    {
        Graphics2D g2d = ( Graphics2D ) g;
        Object old = LafUtils.setupAntialias ( g2d );

        // Variables
        boolean selected = false;
        if ( c instanceof AbstractButton )
        {
            AbstractButton ab = ( AbstractButton ) c;
            ButtonModel bm = ab.getModel ();
            selected = bm.isPressed () || bm.isSelected ();
        }

        // Painting element asuming
        if ( !type.equals ( BreadcrumbElementType.end ) )
        {
            // Shapes
            Shape bs = getBorderShape ( c );
            Shape fs = getFillShape ( c );

            // Shade
            if ( c.isEnabled () )
            {
                LafUtils.drawShade ( g2d, bs, StyleConstants.shadeColor, shadeWidth );
            }

            // Background
            g2d.setPaint ( selected ? selectedBgColor :
                    new GradientPaint ( 0, 0, bgTop, 0, c.getHeight (), bgBottom ) );
            g2d.fill ( fs );

            // Border
            g2d.setPaint ( c.isEnabled () ? borderColor : disabledBorderColor );
            g2d.draw ( bs );
        }
        else
        {
            // Background
            g2d.setPaint ( selected ? selectedBgColor :
                    new GradientPaint ( 0, 0, bgTop, 0, c.getHeight (), bgBottom ) );
            g2d.fillRect ( 0, 0, c.getWidth (), c.getHeight () );
        }

        LafUtils.restoreAntialias ( g2d, old );
    }

    public GeneralPath getBorderShape ( JComponent c )
    {
        GeneralPath gp = new GeneralPath ( GeneralPath.WIND_EVEN_ODD );
        gp.moveTo ( c.getWidth () - overlap - shadeWidth - 1, -1 );
        gp.lineTo ( c.getWidth () - shadeWidth - 1, c.getHeight () / 2 );
        gp.lineTo ( c.getWidth () - overlap - shadeWidth - 1, c.getHeight () );
        return gp;
    }

    public GeneralPath getFillShape ( JComponent c )
    {
        GeneralPath gp = getBorderShape ( c );
        gp.lineTo ( 0, c.getHeight () );
        gp.lineTo ( 0, 0 );
        gp.closePath ();
        return gp;
    }
}