/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.extended.breadcrumb;

import com.alee.extended.layout.BreadcrumbLayout;
import com.alee.laf.panel.WebPanel;

import java.awt.*;

/**
 * User: mgarin Date: 01.03.12 Time: 18:54
 */

public class WebBreadcrumb extends WebPanel
{
    private int overlap = BreadcrumbStyle.overlap;
    private Insets margin = BreadcrumbStyle.margin;

    public WebBreadcrumb ()
    {
        super ();
    }

    public WebBreadcrumb ( boolean decorated )
    {
        super ( decorated );
    }

    protected void initialize ()
    {
        setLayout ( new BreadcrumbLayout ( 10 ) );
    }

    public int getOverlap ()
    {
        return overlap;
    }

    public void setOverlap ( int overlap )
    {
        this.overlap = overlap;
        getBreadcrumbLayout ().setOverlap ( overlap );
        updateElements ();
        updateBreadcrumb ();
    }

    public Insets getMargin ()
    {
        return margin;
    }

    public void setMargin ( Insets margin )
    {
        this.margin = margin;
        updateElements ();
        updateBreadcrumb ();
    }

    private void updateElements ()
    {
        for ( Component component : getComponents () )
        {
            if ( component instanceof WebBreadcrumbButton )
            {
                updateElement ( component );
            }
        }
    }

    public void addElement ( Component element )
    {
        updateElement ( element );
        add ( element );
    }

    public Component getLastElement ()
    {
        return getComponent ( getComponentCount () - 1 );
    }

    public Component[] getElements ()
    {
        return getComponents ();
    }

    public void updateBreadcrumb ()
    {
        revalidate ();
        repaint ();
    }

    private void updateElement ( Component element )
    {
        if ( element instanceof WebBreadcrumbLabel )
        {
            WebBreadcrumbLabel label = ( WebBreadcrumbLabel ) element;
            label.setOverlap ( overlap );
            label.setMargin ( margin );
        }
        else if ( element instanceof WebBreadcrumbButton )
        {
            WebBreadcrumbButton button = ( WebBreadcrumbButton ) element;
            button.setOverlap ( overlap );
            button.setMargin ( margin );
        }
        else if ( element instanceof WebBreadcrumbToggleButton )
        {
            WebBreadcrumbToggleButton button = ( WebBreadcrumbToggleButton ) element;
            button.setOverlap ( overlap );
            button.setMargin ( margin );
        }
    }

    public BreadcrumbLayout getBreadcrumbLayout ()
    {
        return ( BreadcrumbLayout ) super.getLayout ();
    }
}