/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.extended.breadcrumb;

import com.alee.laf.label.WebLabel;

import javax.swing.*;
import java.awt.*;

/**
 * User: mgarin Date: 02.03.12 Time: 12:29
 */

public class WebBreadcrumbLabel extends WebLabel
{
    private BreadcrumbElementPainter backgroundPainter;

    public WebBreadcrumbLabel ( BreadcrumbElementType type )
    {
        super ();
        initialize ( type );
    }

    public WebBreadcrumbLabel ( BreadcrumbElementType type, Icon image )
    {
        super ( image );
        initialize ( type );
    }

    public WebBreadcrumbLabel ( BreadcrumbElementType type, Icon image, int horizontalAlignment )
    {
        super ( image, horizontalAlignment );
        initialize ( type );
    }

    public WebBreadcrumbLabel ( BreadcrumbElementType type, String text )
    {
        super ( text );
        initialize ( type );
    }

    public WebBreadcrumbLabel ( BreadcrumbElementType type, String text, int horizontalAlignment )
    {
        super ( text, horizontalAlignment );
        initialize ( type );
    }

    public WebBreadcrumbLabel ( BreadcrumbElementType type, String text, Icon icon )
    {
        super ( text, icon );
        initialize ( type );
    }

    public WebBreadcrumbLabel ( BreadcrumbElementType type, String text, Icon icon,
                                int horizontalAlignment )
    {
        super ( text, icon, horizontalAlignment );
        initialize ( type );
    }

    private void initialize ( BreadcrumbElementType type )
    {
        setMargin ( BreadcrumbStyle.margin );

        backgroundPainter = new BreadcrumbElementPainter ( type );
        setBackgroundPainter ( backgroundPainter );
    }

    public int getOverlap ()
    {
        return backgroundPainter.getOverlap ();
    }

    public void setOverlap ( int overlap )
    {
        backgroundPainter.setOverlap ( overlap );
        setBackgroundPainter ( backgroundPainter );
    }

    public boolean contains ( int x, int y )
    {
        Shape s = backgroundPainter.getFillShape ( this );
        if ( s != null )
        {
            return s.contains ( x, y );
        }
        else
        {
            return super.contains ( x, y );
        }
    }
}