/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.extended.panel;

import com.alee.extended.background.BackgroundPainter;
import com.alee.extended.background.DefaultBackgroundPainter;
import com.alee.extended.layout.VerticalFlowLayout;
import com.alee.laf.StyleConstants;
import com.alee.laf.panel.WebPanel;
import com.alee.managers.focus.FocusManager;
import com.alee.managers.focus.FocusTracker;

import javax.swing.*;
import java.awt.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.LinkedHashMap;
import java.util.Map;

/**
 * User: mgarin Date: 29.02.12 Time: 16:53
 */

public class WebListPanel extends WebPanel
{
    private Map<Component, WebSelectablePanel> components =
            new LinkedHashMap<Component, WebSelectablePanel> ();

    private Insets elementMargin = new Insets ( 10, 10, 10, 10 );

    public WebListPanel ()
    {
        super ();
    }

    public WebListPanel ( boolean decorated )
    {
        super ( decorated );
    }

    protected void initialize ()
    {
        setLayout ( new VerticalFlowLayout ( VerticalFlowLayout.TOP, 0, 0, true, false ) );
    }

    public void addElement ( Component element )
    {
        // Creating view for component
        WebSelectablePanel elementPanel = new WebSelectablePanel ( components.size () );
        FocusManager.registerFocusTracker ( elementPanel );
        elementPanel.add ( element, BorderLayout.CENTER );
        add ( elementPanel );

        // Saving view for component
        components.put ( element, elementPanel );

        // Updating existing panels
        for ( Component key : components.keySet () )
        {
            components.get ( key ).updateBorder ();
        }
    }

    private class WebSelectablePanel extends WebPanel implements FocusTracker
    {
        private int index;
        private boolean selected = false;
        private BackgroundPainter bg;

        public WebSelectablePanel ( int ind )
        {
            super ();

            this.index = ind;
            updateBorder ();

            bg = new DefaultBackgroundPainter ()
            {
                public boolean isOpaque ()
                {
                    return true;
                }

                public void paintBackground ( Graphics g, Rectangle bounds, JComponent c )
                {
                    Graphics2D g2d = ( Graphics2D ) g;

                    // Background
                    g2d.setPaint (
                            new GradientPaint ( bounds.x, bounds.y, StyleConstants.topBgColor,
                                    bounds.x, bounds.y + bounds.height,
                                    StyleConstants.bottomBgColor ) );
                    g2d.fill ( bounds );

                    // Borders
                    g2d.setPaint ( StyleConstants.darkBorderColor );
                    if ( index > 0 )
                    {
                        g2d.drawLine ( bounds.x, bounds.y, bounds.x + bounds.width - 1, bounds.y );
                    }
                    if ( index < components.size () - 1 )
                    {
                        g2d.drawLine ( bounds.x, bounds.y + bounds.height - 1,
                                bounds.x + bounds.width - 1, bounds.y + bounds.height - 1 );
                    }
                }
            };

            addMouseListener ( new MouseAdapter ()
            {
                public void mousePressed ( MouseEvent e )
                {
                    WebSelectablePanel.this.transferFocus ();
                }
            } );
        }

        public void updateBorder ()
        {
            setMargin ( index == 0 ? elementMargin.top : elementMargin.top + 1, elementMargin.left,
                    index == components.size () - 1 ? elementMargin.bottom :
                            elementMargin.bottom + 1, elementMargin.right );
        }

        public boolean isSelected ()
        {
            return selected;
        }

        public Component getComponent ()
        {
            return this;
        }

        public boolean countChilds ()
        {
            return true;
        }

        public boolean isFocusOwner ()
        {
            return selected;
        }

        public void focusChanged ( boolean focused )
        {
            selected = focused;
            setBackgroundPainter ( focused ? bg : null );
        }
    }
}