/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.laf.text;

import com.alee.extended.background.BackgroundPainter;
import com.alee.laf.StyleConstants;
import com.alee.utils.SwingUtils;

import javax.swing.*;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.basic.BasicTextAreaUI;
import javax.swing.text.JTextComponent;
import java.awt.*;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyAdapter;
import java.awt.event.MouseAdapter;

/**
 * User: mgarin Date: 17.08.11 Time: 23:01
 */

public class WebTextAreaUI extends BasicTextAreaUI
{
    public static final int textMargin = 2;

    private BackgroundPainter backgroundPainter = WebTextAreaStyle.backgroundPainter;

    //    private boolean drawLinesMarkup = true;
    //    private boolean highlightCurrentLine = true;
    //    private int lineNumberColumns = -1;

    private FocusListener focusListener;
    private MouseAdapter mouseAdapter;
    private KeyAdapter keyAdapter;

    public static ComponentUI createUI ( JComponent c )
    {
        return new WebTextAreaUI ();
    }

    public void installUI ( JComponent c )
    {
        super.installUI ( c );

        final JTextComponent component = getComponent ();

        updateBorder ();

        component.setFocusable ( true );
        component.setOpaque ( true );
        component.setBackground ( Color.WHITE );

        component.setSelectionColor ( StyleConstants.textSelectionColor );
        component.setForeground ( Color.BLACK );
        component.setSelectedTextColor ( Color.BLACK );

        component.setCaretColor ( Color.GRAY );

        focusListener = new FocusListener ()
        {
            public void focusLost ( FocusEvent e )
            {
                component.repaint ();
            }

            public void focusGained ( FocusEvent e )
            {
                component.repaint ();
            }
        };
        component.addFocusListener ( focusListener );

        //        mouseAdapter = new MouseAdapter ()
        //        {
        //            public void mousePressed ( MouseEvent e )
        //            {
        //                if ( drawLinesMarkup )
        //                {
        //                    component.repaint ();
        //                }
        //            }
        //
        //            public void mouseDragged ( MouseEvent e )
        //            {
        //                if ( drawLinesMarkup )
        //                {
        //                    component.repaint ();
        //                }
        //            }
        //
        //            public void mouseReleased ( MouseEvent e )
        //            {
        //                if ( drawLinesMarkup )
        //                {
        //                    component.repaint ();
        //                }
        //            }
        //
        //            //            public void mouseEntered ( MouseEvent e )
        //            //            {
        //            //                textField.repaint ();
        //            //                mouseover = true;
        //            //            }
        //            //
        //            //            public void mouseExited ( MouseEvent e )
        //            //            {
        //            //                textField.repaint ();
        //            //                mouseover = false;
        //            //            }
        //        };
        //        component.addMouseListener ( mouseAdapter );
        //        component.addMouseMotionListener ( mouseAdapter );
        //
        //        keyAdapter = new KeyAdapter ()
        //        {
        //            public void keyPressed ( KeyEvent e )
        //            {
        //                if ( drawLinesMarkup )
        //                {
        //                    updateBorder ();
        //                    component.repaint ();
        //                }
        //            }
        //        };
        //        component.addKeyListener ( keyAdapter );
    }

    public void uninstallUI ( JComponent c )
    {
        JTextComponent component = getComponent ();
        component.removeFocusListener ( focusListener );
        //        component.removeMouseListener ( mouseAdapter );
        //        component.removeMouseMotionListener ( mouseAdapter );
        //        component.removeKeyListener ( keyAdapter );

        super.uninstallUI ( c );
    }

    public BackgroundPainter getBackgroundPainter ()
    {
        return backgroundPainter;
    }

    public void setBackgroundPainter ( BackgroundPainter backgroundPainter )
    {
        this.backgroundPainter = backgroundPainter;
        getComponent ().setOpaque ( backgroundPainter == null || backgroundPainter.isOpaque () );
        updateBorder ();
    }

    private void updateBorder ()
    {
        JTextComponent component = getComponent ();
        if ( component != null )
        {
            //            int additionalSpace = getMarkupSpace ( component );
            if ( backgroundPainter != null )
            {
                // Background insets
                Insets margin = backgroundPainter.getBackgroundMargin ( component );
                component.setBorder ( BorderFactory
                        .createEmptyBorder ( margin.top, /*additionalSpace +*/ margin.left,
                                margin.bottom, margin.right ) );
            }
            else
            {
                // Empty insets
                component.setBorder ( BorderFactory
                        .createEmptyBorder ( textMargin, /*additionalSpace +*/ textMargin,
                                textMargin, textMargin ) );
            }
        }
    }

    protected void paintBackground ( Graphics g )
    {
        //
    }

    protected void paintSafely ( Graphics g )
    {
        Graphics2D g2d = ( Graphics2D ) g;
        JTextComponent c = getComponent ();

        if ( c.isOpaque () && ( backgroundPainter == null || !backgroundPainter.isOpaque () ) )
        {
            // Paint default background
            g2d.setPaint ( c.getBackground () );
            g2d.fillRect ( 0, 0, c.getWidth (), c.getHeight () );
        }

        if ( backgroundPainter != null )
        {
            // Use background painter instead of default UI graphics
            backgroundPainter.paintBackground ( g, SwingUtils.size ( c ), c );
        }

        //        if ( drawLinesMarkup )
        //        {
        //        FontMetrics fm = g2d.getFontMetrics ();
        //            Insets bi = c.getInsets ();
        //            int space = getMarkupSpace ( c );
        //
        //            g2d.setPaint ( new GradientPaint ( bi.left - space, 0, Color.WHITE, bi.left, 0,
        //                    new Color ( 235, 235, 235 ) ) );
        //            g2d.fillRect ( bi.left - space - textMargin, bi.top - textMargin, space,
        //                    c.getHeight () - bi.top - bi.bottom + textMargin * 2 );
        //            g2d.setPaint ( StyleConstants.borderColor );
        //            g2d.drawLine ( bi.left - textMargin - 1, bi.top - textMargin, bi.left - textMargin - 1,
        //                    c.getHeight () - bi.bottom + textMargin );
        //
        //
        //            int caretRow = getCaretRow ( c );
        //            int rowHeight = g.getFontMetrics ().getHeight ();
        //            g2d.setPaint ( new Color ( 255, 255, 0, 55 ) );
        //            g2d.fillRect ( bi.left - space - textMargin, bi.top + ( caretRow ) * rowHeight,
        //                    c.getWidth () - ( bi.left - space - textMargin ) - ( bi.right - textMargin ),
        //                    rowHeight );
        //
        //            g2d.setColor ( Color.DARK_GRAY );
        //            int rowCount = getRowCount ( c );
        //            for ( int i = 0; i < c.getHeight () / rowHeight && i < rowCount; i++ )
        //            {
        //                String number = "" + ( i + 1 );
        //                g2d.drawString ( number, bi.left - textMargin - 3 - fm.stringWidth ( number ),
        //                        bi.top - textMargin + ( i + 1 ) * rowHeight - rowHeight / 2 +
        //                                fm.getAscent () / 2 );
        //            }
        //        }

        super.paintSafely ( g );
    }

    //    private int getMarkupSpace ( JTextComponent component )
    //    {
    //        int additionalSpace = 0;
    //        if ( drawLinesMarkup )
    //        {
    //            int lineNumberColumns = getlineNumberColumns ();
    //            FontMetrics fm = component.getFontMetrics ( component.getFont () );
    //            additionalSpace += 3 + fm.stringWidth ( "O" ) * lineNumberColumns + 3 + 3;
    //        }
    //        return additionalSpace;
    //    }
    //
    //    public int getlineNumberColumns ()
    //    {
    //        return lineNumberColumns > 0 ? lineNumberColumns :
    //                ( "" + getRowCount ( getComponent () ) ).length ();
    //    }
    //
    //    private int getCaretRow ( JTextComponent c )
    //    {
    //        int caretRow = 0;
    //        String text = c.getText ().substring ( 0, c.getCaretPosition () );
    //        String separator = "\n";
    //        int index;
    //        while ( ( index = text.indexOf ( separator ) ) != -1 )
    //        {
    //            caretRow++;
    //            text = text.substring ( index + separator.length () );
    //        }
    //        return caretRow;
    //    }
    //
    //    public int getRowCount ( JTextComponent c )
    //    {
    //        int count = 1;
    //        String text = c.getText ();
    //        String separator = "\n";
    //        int index;
    //        while ( ( index = text.indexOf ( separator ) ) != -1 )
    //        {
    //            count++;
    //            text = text.substring ( index + separator.length () );
    //        }
    //        return count;
    //    }

    public Dimension getPreferredSize ( JComponent c )
    {
        Dimension ps = super.getPreferredSize ( c );
        if ( backgroundPainter != null )
        {
            if ( c.getLayout () != null )
            {
                ps = SwingUtils.max ( ps, c.getLayout ().preferredLayoutSize ( c ) );
            }
            ps = SwingUtils.max ( ps, backgroundPainter.getBackgroundPreferredSize ( c ) );
        }
        return ps;
    }
}
